#!/bin/bash

## Author Oriol Filter
# Date 27/03/2022

## Send scripts/files to the servers

### COLORS

_RED="\033[1;31m"
_GREEN="\033[1;32m"
_RESET="\033[0m"
_YELLOW="\033[0;33m"

### VAR

_USERS="$(cat ./users)"
_FILES="$(cat ./files_to_send)"
_SSH_TIMOUT=5


### Main

printf "Sending files to /tmp\n"
for username in $_USERS
do
  printf "\033[0;33m ${username}${_RESET}\n"

  for hostname in $(cat "${username}/h")
  do
    printf "\033[1;34m   ${hostname}${_RESET}\n"
    for _F in $_FILES
    do
      result=""
      printf "\t${_F}: "
      sshpass -f "./${username}/p" scp -q -o StrictHostKeyChecking=no -o ConnectTimeout=${_SSH_TIMOUT} "$_F" "${username}@${hostname}:/tmp/" && result="true"
      if [[ "$result" ]]
        then
          printf "${_GREEN}YES${_RESET}\n"
        else
          printf "${_RED}No${_RESET}\n";
      fi
    done
    printf "   -----------\n"
  done
done


# Copy files to proxy host
PROXY_IN_RESLTS=$(awk -F ';' '{print $2}' ./.proxy_results | sort  | uniq)

for _proxy in $PROXY_IN_RESLTS
do
  printf ">> ${_YELLOW}${_proxy}:${_RESET}\n"
  printf "" > "./_${_proxy}/result"
  proxy_username="$(cat ./_${_proxy}/u)"
  for _F in $_FILES
  do
    result=""
    sshpass -f "./${_proxy}/p" scp -q -o StrictHostKeyChecking=no -o ConnectTimeout=${_SSH_TIMOUT} "$_F" "${proxy_username}@${_proxy}:/tmp" && result="true";
    printf "\t${_F}: "
    if [[ "$result" ]]
      then
        printf "${_GREEN}YES${_RESET}\n"
      else
        printf "${_RED}No${_RESET}\n";
    fi
  done
  printf "   -----------\n"
done



printf "Sending files from proxy hosts\n"
# Copy files from proxy to remote
PROXY_SORTED_LINES=$(sort -t ';' -k2 ./.proxy_results)

#for _LINE in $PROXY_SORTED_LINES
while IFS= read -r _LINE
do
    _proxy=$(awk -F ';' '{print $2}' <<< "$_LINE")
    hostname=$(awk -F ';' '{print $1}' <<< "$_LINE")
    username=$(awk -F ';' '{print $3}' <<< "$_LINE")
    proxy_username=$(cat "./_${_proxy}/u")
    printf "%s  -->\t%s\n" "$_proxy" "$hostname"
    for _F in $_FILES
    do
      result=""
      cat "./${username}/p" | sshpass -f "./${_proxy}/p" ssh -q -o StrictHostKeyChecking=no -o ConnectTimeout=${_SSH_TIMOUT} -T "${proxy_username}@${_proxy}" "sshpass -f /dev/stdin scp -q -o StrictHostKeyChecking=no -o ConnectTimeout=${_SSH_TIMOUT} /tmp/${_F} \"${username}@${hostname}\" && echo true" &> /dev/null && result="true";
      printf "\t${_F}: "
      if [[ "$result" ]]
        then
          printf "${_GREEN}YES${_RESET}\n"
        else
          printf "${_RED}No${_RESET}\n";
      fi
    done
    printf "   -----------\n"
done <<< "$PROXY_SORTED_LINES"


printf "${_GREEN}DONE${_RESET}\n"
